# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import duration_pb2  # type: ignore
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from google.type import datetime_pb2  # type: ignore
from google.type import dayofweek_pb2  # type: ignore
from google.type import timeofday_pb2  # type: ignore
import proto  # type: ignore

from google.cloud.osconfig_v1.types import patch_jobs

__protobuf__ = proto.module(
    package="google.cloud.osconfig.v1",
    manifest={
        "PatchDeployment",
        "OneTimeSchedule",
        "RecurringSchedule",
        "WeeklySchedule",
        "MonthlySchedule",
        "WeekDayOfMonth",
        "CreatePatchDeploymentRequest",
        "GetPatchDeploymentRequest",
        "ListPatchDeploymentsRequest",
        "ListPatchDeploymentsResponse",
        "DeletePatchDeploymentRequest",
        "UpdatePatchDeploymentRequest",
        "PausePatchDeploymentRequest",
        "ResumePatchDeploymentRequest",
    },
)


class PatchDeployment(proto.Message):
    r"""Patch deployments are configurations that individual patch jobs use
    to complete a patch. These configurations include instance filter,
    package repository settings, and a schedule. For more information
    about creating and managing patch deployments, see `Scheduling patch
    jobs <https://cloud.google.com/compute/docs/os-patch-management/schedule-patch-jobs>`__.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            Unique name for the patch deployment resource in a project.
            The patch deployment name is in the form:
            ``projects/{project_id}/patchDeployments/{patch_deployment_id}``.
            This field is ignored when you create a new patch
            deployment.
        description (str):
            Optional. Description of the patch
            deployment. Length of the description is limited
            to 1024 characters.
        instance_filter (google.cloud.osconfig_v1.types.PatchInstanceFilter):
            Required. VM instances to patch.
        patch_config (google.cloud.osconfig_v1.types.PatchConfig):
            Optional. Patch configuration that is
            applied.
        duration (google.protobuf.duration_pb2.Duration):
            Optional. Duration of the patch. After the
            duration ends, the patch times out.
        one_time_schedule (google.cloud.osconfig_v1.types.OneTimeSchedule):
            Required. Schedule a one-time execution.

            This field is a member of `oneof`_ ``schedule``.
        recurring_schedule (google.cloud.osconfig_v1.types.RecurringSchedule):
            Required. Schedule recurring executions.

            This field is a member of `oneof`_ ``schedule``.
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time the patch deployment was created.
            Timestamp is in
            `RFC3339 <https://www.ietf.org/rfc/rfc3339.txt>`__ text
            format.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Time the patch deployment was last updated.
            Timestamp is in
            `RFC3339 <https://www.ietf.org/rfc/rfc3339.txt>`__ text
            format.
        last_execute_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The last time a patch job was started by this
            deployment. Timestamp is in
            `RFC3339 <https://www.ietf.org/rfc/rfc3339.txt>`__ text
            format.
        rollout (google.cloud.osconfig_v1.types.PatchRollout):
            Optional. Rollout strategy of the patch job.
        state (google.cloud.osconfig_v1.types.PatchDeployment.State):
            Output only. Current state of the patch
            deployment.
    """

    class State(proto.Enum):
        r"""Represents state of patch peployment.

        Values:
            STATE_UNSPECIFIED (0):
                The default value. This value is used if the
                state is omitted.
            ACTIVE (1):
                Active value means that patch deployment
                generates Patch Jobs.
            PAUSED (2):
                Paused value means that patch deployment does
                not generate Patch jobs. Requires user action to
                move in and out from this state.
        """
        STATE_UNSPECIFIED = 0
        ACTIVE = 1
        PAUSED = 2

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    description: str = proto.Field(
        proto.STRING,
        number=2,
    )
    instance_filter: patch_jobs.PatchInstanceFilter = proto.Field(
        proto.MESSAGE,
        number=3,
        message=patch_jobs.PatchInstanceFilter,
    )
    patch_config: patch_jobs.PatchConfig = proto.Field(
        proto.MESSAGE,
        number=4,
        message=patch_jobs.PatchConfig,
    )
    duration: duration_pb2.Duration = proto.Field(
        proto.MESSAGE,
        number=5,
        message=duration_pb2.Duration,
    )
    one_time_schedule: "OneTimeSchedule" = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="schedule",
        message="OneTimeSchedule",
    )
    recurring_schedule: "RecurringSchedule" = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="schedule",
        message="RecurringSchedule",
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=8,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=9,
        message=timestamp_pb2.Timestamp,
    )
    last_execute_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=10,
        message=timestamp_pb2.Timestamp,
    )
    rollout: patch_jobs.PatchRollout = proto.Field(
        proto.MESSAGE,
        number=11,
        message=patch_jobs.PatchRollout,
    )
    state: State = proto.Field(
        proto.ENUM,
        number=12,
        enum=State,
    )


class OneTimeSchedule(proto.Message):
    r"""Sets the time for a one time patch deployment. Timestamp is in
    `RFC3339 <https://www.ietf.org/rfc/rfc3339.txt>`__ text format.

    Attributes:
        execute_time (google.protobuf.timestamp_pb2.Timestamp):
            Required. The desired patch job execution
            time.
    """

    execute_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=1,
        message=timestamp_pb2.Timestamp,
    )


class RecurringSchedule(proto.Message):
    r"""Sets the time for recurring patch deployments.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        time_zone (google.type.datetime_pb2.TimeZone):
            Required. Defines the time zone that ``time_of_day`` is
            relative to. The rules for daylight saving time are
            determined by the chosen time zone.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            Optional. The time that the recurring schedule becomes
            effective. Defaults to ``create_time`` of the patch
            deployment.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            Optional. The end time at which a recurring
            patch deployment schedule is no longer active.
        time_of_day (google.type.timeofday_pb2.TimeOfDay):
            Required. Time of the day to run a recurring
            deployment.
        frequency (google.cloud.osconfig_v1.types.RecurringSchedule.Frequency):
            Required. The frequency unit of this
            recurring schedule.
        weekly (google.cloud.osconfig_v1.types.WeeklySchedule):
            Required. Schedule with weekly executions.

            This field is a member of `oneof`_ ``schedule_config``.
        monthly (google.cloud.osconfig_v1.types.MonthlySchedule):
            Required. Schedule with monthly executions.

            This field is a member of `oneof`_ ``schedule_config``.
        last_execute_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time the last patch job ran
            successfully.
        next_execute_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The time the next patch job is
            scheduled to run.
    """

    class Frequency(proto.Enum):
        r"""Specifies the frequency of the recurring patch deployments.

        Values:
            FREQUENCY_UNSPECIFIED (0):
                Invalid. A frequency must be specified.
            WEEKLY (1):
                Indicates that the frequency of recurrence
                should be expressed in terms of weeks.
            MONTHLY (2):
                Indicates that the frequency of recurrence
                should be expressed in terms of months.
            DAILY (3):
                Indicates that the frequency of recurrence
                should be expressed in terms of days.
        """
        FREQUENCY_UNSPECIFIED = 0
        WEEKLY = 1
        MONTHLY = 2
        DAILY = 3

    time_zone: datetime_pb2.TimeZone = proto.Field(
        proto.MESSAGE,
        number=1,
        message=datetime_pb2.TimeZone,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    time_of_day: timeofday_pb2.TimeOfDay = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timeofday_pb2.TimeOfDay,
    )
    frequency: Frequency = proto.Field(
        proto.ENUM,
        number=5,
        enum=Frequency,
    )
    weekly: "WeeklySchedule" = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="schedule_config",
        message="WeeklySchedule",
    )
    monthly: "MonthlySchedule" = proto.Field(
        proto.MESSAGE,
        number=7,
        oneof="schedule_config",
        message="MonthlySchedule",
    )
    last_execute_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=9,
        message=timestamp_pb2.Timestamp,
    )
    next_execute_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=10,
        message=timestamp_pb2.Timestamp,
    )


class WeeklySchedule(proto.Message):
    r"""Represents a weekly schedule.

    Attributes:
        day_of_week (google.type.dayofweek_pb2.DayOfWeek):
            Required. Day of the week.
    """

    day_of_week: dayofweek_pb2.DayOfWeek = proto.Field(
        proto.ENUM,
        number=1,
        enum=dayofweek_pb2.DayOfWeek,
    )


class MonthlySchedule(proto.Message):
    r"""Represents a monthly schedule. An example of a valid monthly
    schedule is "on the third Tuesday of the month" or "on the 15th
    of the month".

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        week_day_of_month (google.cloud.osconfig_v1.types.WeekDayOfMonth):
            Required. Week day in a month.

            This field is a member of `oneof`_ ``day_of_month``.
        month_day (int):
            Required. One day of the month. 1-31
            indicates the 1st to the 31st day. -1 indicates
            the last day of the month. Months without the
            target day will be skipped. For example, a
            schedule to run "every month on the 31st" will
            not run in February, April, June, etc.

            This field is a member of `oneof`_ ``day_of_month``.
    """

    week_day_of_month: "WeekDayOfMonth" = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="day_of_month",
        message="WeekDayOfMonth",
    )
    month_day: int = proto.Field(
        proto.INT32,
        number=2,
        oneof="day_of_month",
    )


class WeekDayOfMonth(proto.Message):
    r"""Represents one week day in a month. An example is "the 4th
    Sunday".

    Attributes:
        week_ordinal (int):
            Required. Week number in a month. 1-4
            indicates the 1st to 4th week of the month. -1
            indicates the last week of the month.
        day_of_week (google.type.dayofweek_pb2.DayOfWeek):
            Required. A day of the week.
        day_offset (int):
            Optional. Represents the number of days before or after the
            given week day of month that the patch deployment is
            scheduled for. For example if ``week_ordinal`` and
            ``day_of_week`` values point to the second day of the month
            and this ``day_offset`` value is set to ``3``, the patch
            deployment takes place three days after the second Tuesday
            of the month. If this value is negative, for example -5, the
            patches are deployed five days before before the second
            Tuesday of the month. Allowed values are in range [-30, 30].
    """

    week_ordinal: int = proto.Field(
        proto.INT32,
        number=1,
    )
    day_of_week: dayofweek_pb2.DayOfWeek = proto.Field(
        proto.ENUM,
        number=2,
        enum=dayofweek_pb2.DayOfWeek,
    )
    day_offset: int = proto.Field(
        proto.INT32,
        number=3,
    )


class CreatePatchDeploymentRequest(proto.Message):
    r"""A request message for creating a patch deployment.

    Attributes:
        parent (str):
            Required. The project to apply this patch deployment to in
            the form ``projects/*``.
        patch_deployment_id (str):
            Required. A name for the patch deployment in the project.
            When creating a name the following rules apply:

            -  Must contain only lowercase letters, numbers, and
               hyphens.
            -  Must start with a letter.
            -  Must be between 1-63 characters.
            -  Must end with a number or a letter.
            -  Must be unique within the project.
        patch_deployment (google.cloud.osconfig_v1.types.PatchDeployment):
            Required. The patch deployment to create.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    patch_deployment_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    patch_deployment: "PatchDeployment" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="PatchDeployment",
    )


class GetPatchDeploymentRequest(proto.Message):
    r"""A request message for retrieving a patch deployment.

    Attributes:
        name (str):
            Required. The resource name of the patch deployment in the
            form ``projects/*/patchDeployments/*``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListPatchDeploymentsRequest(proto.Message):
    r"""A request message for listing patch deployments.

    Attributes:
        parent (str):
            Required. The resource name of the parent in the form
            ``projects/*``.
        page_size (int):
            Optional. The maximum number of patch
            deployments to return. Default is 100.
        page_token (str):
            Optional. A pagination token returned from a
            previous call to ListPatchDeployments that
            indicates where this listing should continue
            from.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListPatchDeploymentsResponse(proto.Message):
    r"""A response message for listing patch deployments.

    Attributes:
        patch_deployments (MutableSequence[google.cloud.osconfig_v1.types.PatchDeployment]):
            The list of patch deployments.
        next_page_token (str):
            A pagination token that can be used to get
            the next page of patch deployments.
    """

    @property
    def raw_page(self):
        return self

    patch_deployments: MutableSequence["PatchDeployment"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="PatchDeployment",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class DeletePatchDeploymentRequest(proto.Message):
    r"""A request message for deleting a patch deployment.

    Attributes:
        name (str):
            Required. The resource name of the patch deployment in the
            form ``projects/*/patchDeployments/*``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class UpdatePatchDeploymentRequest(proto.Message):
    r"""A request message for updating a patch deployment.

    Attributes:
        patch_deployment (google.cloud.osconfig_v1.types.PatchDeployment):
            Required. The patch deployment to Update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. Field mask that controls which
            fields of the patch deployment should be
            updated.
    """

    patch_deployment: "PatchDeployment" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="PatchDeployment",
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class PausePatchDeploymentRequest(proto.Message):
    r"""A request message for pausing a patch deployment.

    Attributes:
        name (str):
            Required. The resource name of the patch deployment in the
            form ``projects/*/patchDeployments/*``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ResumePatchDeploymentRequest(proto.Message):
    r"""A request message for resuming a patch deployment.

    Attributes:
        name (str):
            Required. The resource name of the patch deployment in the
            form ``projects/*/patchDeployments/*``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import dataclasses
import json  # type: ignore
import re
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings

from google.api_core import gapic_v1, path_template, rest_helpers, rest_streaming
from google.api_core import exceptions as core_exceptions
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.protobuf import json_format
import grpc  # type: ignore
from requests import __version__ as requests_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore


from google.protobuf import empty_pb2  # type: ignore

from google.cloud.osconfig_v1.types import patch_deployments, patch_jobs

from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO
from .base import OsConfigServiceTransport

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=requests_version,
)


class OsConfigServiceRestInterceptor:
    """Interceptor for OsConfigService.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the OsConfigServiceRestTransport.

    .. code-block:: python
        class MyCustomOsConfigServiceInterceptor(OsConfigServiceRestInterceptor):
            def pre_cancel_patch_job(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_cancel_patch_job(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_create_patch_deployment(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_create_patch_deployment(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_delete_patch_deployment(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def pre_execute_patch_job(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_execute_patch_job(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_get_patch_deployment(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_patch_deployment(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_get_patch_job(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_patch_job(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_list_patch_deployments(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_list_patch_deployments(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_list_patch_job_instance_details(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_list_patch_job_instance_details(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_list_patch_jobs(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_list_patch_jobs(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_pause_patch_deployment(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_pause_patch_deployment(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_resume_patch_deployment(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_resume_patch_deployment(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_update_patch_deployment(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_update_patch_deployment(self, response):
                logging.log(f"Received response: {response}")
                return response

        transport = OsConfigServiceRestTransport(interceptor=MyCustomOsConfigServiceInterceptor())
        client = OsConfigServiceClient(transport=transport)


    """

    def pre_cancel_patch_job(
        self,
        request: patch_jobs.CancelPatchJobRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[patch_jobs.CancelPatchJobRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for cancel_patch_job

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_cancel_patch_job(
        self, response: patch_jobs.PatchJob
    ) -> patch_jobs.PatchJob:
        """Post-rpc interceptor for cancel_patch_job

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response

    def pre_create_patch_deployment(
        self,
        request: patch_deployments.CreatePatchDeploymentRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[
        patch_deployments.CreatePatchDeploymentRequest, Sequence[Tuple[str, str]]
    ]:
        """Pre-rpc interceptor for create_patch_deployment

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_create_patch_deployment(
        self, response: patch_deployments.PatchDeployment
    ) -> patch_deployments.PatchDeployment:
        """Post-rpc interceptor for create_patch_deployment

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response

    def pre_delete_patch_deployment(
        self,
        request: patch_deployments.DeletePatchDeploymentRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[
        patch_deployments.DeletePatchDeploymentRequest, Sequence[Tuple[str, str]]
    ]:
        """Pre-rpc interceptor for delete_patch_deployment

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def pre_execute_patch_job(
        self,
        request: patch_jobs.ExecutePatchJobRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[patch_jobs.ExecutePatchJobRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for execute_patch_job

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_execute_patch_job(
        self, response: patch_jobs.PatchJob
    ) -> patch_jobs.PatchJob:
        """Post-rpc interceptor for execute_patch_job

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response

    def pre_get_patch_deployment(
        self,
        request: patch_deployments.GetPatchDeploymentRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[patch_deployments.GetPatchDeploymentRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for get_patch_deployment

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_get_patch_deployment(
        self, response: patch_deployments.PatchDeployment
    ) -> patch_deployments.PatchDeployment:
        """Post-rpc interceptor for get_patch_deployment

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response

    def pre_get_patch_job(
        self,
        request: patch_jobs.GetPatchJobRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[patch_jobs.GetPatchJobRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for get_patch_job

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_get_patch_job(self, response: patch_jobs.PatchJob) -> patch_jobs.PatchJob:
        """Post-rpc interceptor for get_patch_job

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response

    def pre_list_patch_deployments(
        self,
        request: patch_deployments.ListPatchDeploymentsRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[
        patch_deployments.ListPatchDeploymentsRequest, Sequence[Tuple[str, str]]
    ]:
        """Pre-rpc interceptor for list_patch_deployments

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_list_patch_deployments(
        self, response: patch_deployments.ListPatchDeploymentsResponse
    ) -> patch_deployments.ListPatchDeploymentsResponse:
        """Post-rpc interceptor for list_patch_deployments

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response

    def pre_list_patch_job_instance_details(
        self,
        request: patch_jobs.ListPatchJobInstanceDetailsRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[
        patch_jobs.ListPatchJobInstanceDetailsRequest, Sequence[Tuple[str, str]]
    ]:
        """Pre-rpc interceptor for list_patch_job_instance_details

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_list_patch_job_instance_details(
        self, response: patch_jobs.ListPatchJobInstanceDetailsResponse
    ) -> patch_jobs.ListPatchJobInstanceDetailsResponse:
        """Post-rpc interceptor for list_patch_job_instance_details

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response

    def pre_list_patch_jobs(
        self,
        request: patch_jobs.ListPatchJobsRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[patch_jobs.ListPatchJobsRequest, Sequence[Tuple[str, str]]]:
        """Pre-rpc interceptor for list_patch_jobs

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_list_patch_jobs(
        self, response: patch_jobs.ListPatchJobsResponse
    ) -> patch_jobs.ListPatchJobsResponse:
        """Post-rpc interceptor for list_patch_jobs

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response

    def pre_pause_patch_deployment(
        self,
        request: patch_deployments.PausePatchDeploymentRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[
        patch_deployments.PausePatchDeploymentRequest, Sequence[Tuple[str, str]]
    ]:
        """Pre-rpc interceptor for pause_patch_deployment

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_pause_patch_deployment(
        self, response: patch_deployments.PatchDeployment
    ) -> patch_deployments.PatchDeployment:
        """Post-rpc interceptor for pause_patch_deployment

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response

    def pre_resume_patch_deployment(
        self,
        request: patch_deployments.ResumePatchDeploymentRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[
        patch_deployments.ResumePatchDeploymentRequest, Sequence[Tuple[str, str]]
    ]:
        """Pre-rpc interceptor for resume_patch_deployment

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_resume_patch_deployment(
        self, response: patch_deployments.PatchDeployment
    ) -> patch_deployments.PatchDeployment:
        """Post-rpc interceptor for resume_patch_deployment

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response

    def pre_update_patch_deployment(
        self,
        request: patch_deployments.UpdatePatchDeploymentRequest,
        metadata: Sequence[Tuple[str, str]],
    ) -> Tuple[
        patch_deployments.UpdatePatchDeploymentRequest, Sequence[Tuple[str, str]]
    ]:
        """Pre-rpc interceptor for update_patch_deployment

        Override in a subclass to manipulate the request or metadata
        before they are sent to the OsConfigService server.
        """
        return request, metadata

    def post_update_patch_deployment(
        self, response: patch_deployments.PatchDeployment
    ) -> patch_deployments.PatchDeployment:
        """Post-rpc interceptor for update_patch_deployment

        Override in a subclass to manipulate the response
        after it is returned by the OsConfigService server but before
        it is returned to user code.
        """
        return response


@dataclasses.dataclass
class OsConfigServiceRestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: OsConfigServiceRestInterceptor


class OsConfigServiceRestTransport(OsConfigServiceTransport):
    """REST backend transport for OsConfigService.

    OS Config API
    The OS Config service is a server-side component that you can
    use to manage package installations and patch jobs for virtual
    machine instances.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1

    """

    def __init__(
        self,
        *,
        host: str = "osconfig.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        interceptor: Optional[OsConfigServiceRestInterceptor] = None,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.

            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                certificate to configure mutual TLS HTTP channel. It is ignored
                if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        maybe_url_match = re.match("^(?P<scheme>http(?:s)?://)?(?P<host>.*)$", host)
        if maybe_url_match is None:
            raise ValueError(
                f"Unexpected hostname structure: {host}"
            )  # pragma: NO COVER

        url_match_items = maybe_url_match.groupdict()

        host = f"{url_scheme}://{host}" if not url_match_items["scheme"] else host

        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            api_audience=api_audience,
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST
        )
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = interceptor or OsConfigServiceRestInterceptor()
        self._prep_wrapped_messages(client_info)

    class _CancelPatchJob(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("CancelPatchJob")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_jobs.CancelPatchJobRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_jobs.PatchJob:
            r"""Call the cancel patch job method over HTTP.

            Args:
                request (~.patch_jobs.CancelPatchJobRequest):
                    The request object. Message for canceling a patch job.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.patch_jobs.PatchJob:
                    A high level representation of a patch job that is
                either in progress or has completed.

                Instance details are not included in the job. To
                paginate through instance details, use
                ListPatchJobInstanceDetails.

                For more information about patch jobs, see `Creating
                patch
                jobs <https://cloud.google.com/compute/docs/os-patch-management/create-patch-job>`__.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1/{name=projects/*/patchJobs/*}:cancel",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_cancel_patch_job(
                request, metadata
            )
            pb_request = patch_jobs.CancelPatchJobRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_jobs.PatchJob()
            pb_resp = patch_jobs.PatchJob.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_cancel_patch_job(resp)
            return resp

    class _CreatePatchDeployment(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("CreatePatchDeployment")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {
            "patchDeploymentId": "",
        }

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_deployments.CreatePatchDeploymentRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_deployments.PatchDeployment:
            r"""Call the create patch deployment method over HTTP.

            Args:
                request (~.patch_deployments.CreatePatchDeploymentRequest):
                    The request object. A request message for creating a
                patch deployment.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.patch_deployments.PatchDeployment:
                    Patch deployments are configurations that individual
                patch jobs use to complete a patch. These configurations
                include instance filter, package repository settings,
                and a schedule. For more information about creating and
                managing patch deployments, see `Scheduling patch
                jobs <https://cloud.google.com/compute/docs/os-patch-management/schedule-patch-jobs>`__.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1/{parent=projects/*}/patchDeployments",
                    "body": "patch_deployment",
                },
            ]
            request, metadata = self._interceptor.pre_create_patch_deployment(
                request, metadata
            )
            pb_request = patch_deployments.CreatePatchDeploymentRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_deployments.PatchDeployment()
            pb_resp = patch_deployments.PatchDeployment.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_create_patch_deployment(resp)
            return resp

    class _DeletePatchDeployment(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("DeletePatchDeployment")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_deployments.DeletePatchDeploymentRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ):
            r"""Call the delete patch deployment method over HTTP.

            Args:
                request (~.patch_deployments.DeletePatchDeploymentRequest):
                    The request object. A request message for deleting a
                patch deployment.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.
            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "delete",
                    "uri": "/v1/{name=projects/*/patchDeployments/*}",
                },
            ]
            request, metadata = self._interceptor.pre_delete_patch_deployment(
                request, metadata
            )
            pb_request = patch_deployments.DeletePatchDeploymentRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

    class _ExecutePatchJob(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("ExecutePatchJob")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_jobs.ExecutePatchJobRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_jobs.PatchJob:
            r"""Call the execute patch job method over HTTP.

            Args:
                request (~.patch_jobs.ExecutePatchJobRequest):
                    The request object. A request message to initiate
                patching across Compute Engine
                instances.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.patch_jobs.PatchJob:
                    A high level representation of a patch job that is
                either in progress or has completed.

                Instance details are not included in the job. To
                paginate through instance details, use
                ListPatchJobInstanceDetails.

                For more information about patch jobs, see `Creating
                patch
                jobs <https://cloud.google.com/compute/docs/os-patch-management/create-patch-job>`__.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1/{parent=projects/*}/patchJobs:execute",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_execute_patch_job(
                request, metadata
            )
            pb_request = patch_jobs.ExecutePatchJobRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_jobs.PatchJob()
            pb_resp = patch_jobs.PatchJob.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_execute_patch_job(resp)
            return resp

    class _GetPatchDeployment(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("GetPatchDeployment")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_deployments.GetPatchDeploymentRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_deployments.PatchDeployment:
            r"""Call the get patch deployment method over HTTP.

            Args:
                request (~.patch_deployments.GetPatchDeploymentRequest):
                    The request object. A request message for retrieving a
                patch deployment.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.patch_deployments.PatchDeployment:
                    Patch deployments are configurations that individual
                patch jobs use to complete a patch. These configurations
                include instance filter, package repository settings,
                and a schedule. For more information about creating and
                managing patch deployments, see `Scheduling patch
                jobs <https://cloud.google.com/compute/docs/os-patch-management/schedule-patch-jobs>`__.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1/{name=projects/*/patchDeployments/*}",
                },
            ]
            request, metadata = self._interceptor.pre_get_patch_deployment(
                request, metadata
            )
            pb_request = patch_deployments.GetPatchDeploymentRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_deployments.PatchDeployment()
            pb_resp = patch_deployments.PatchDeployment.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_get_patch_deployment(resp)
            return resp

    class _GetPatchJob(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("GetPatchJob")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_jobs.GetPatchJobRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_jobs.PatchJob:
            r"""Call the get patch job method over HTTP.

            Args:
                request (~.patch_jobs.GetPatchJobRequest):
                    The request object. Request to get an active or completed
                patch job.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.patch_jobs.PatchJob:
                    A high level representation of a patch job that is
                either in progress or has completed.

                Instance details are not included in the job. To
                paginate through instance details, use
                ListPatchJobInstanceDetails.

                For more information about patch jobs, see `Creating
                patch
                jobs <https://cloud.google.com/compute/docs/os-patch-management/create-patch-job>`__.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1/{name=projects/*/patchJobs/*}",
                },
            ]
            request, metadata = self._interceptor.pre_get_patch_job(request, metadata)
            pb_request = patch_jobs.GetPatchJobRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_jobs.PatchJob()
            pb_resp = patch_jobs.PatchJob.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_get_patch_job(resp)
            return resp

    class _ListPatchDeployments(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("ListPatchDeployments")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_deployments.ListPatchDeploymentsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_deployments.ListPatchDeploymentsResponse:
            r"""Call the list patch deployments method over HTTP.

            Args:
                request (~.patch_deployments.ListPatchDeploymentsRequest):
                    The request object. A request message for listing patch
                deployments.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.patch_deployments.ListPatchDeploymentsResponse:
                    A response message for listing patch
                deployments.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1/{parent=projects/*}/patchDeployments",
                },
            ]
            request, metadata = self._interceptor.pre_list_patch_deployments(
                request, metadata
            )
            pb_request = patch_deployments.ListPatchDeploymentsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_deployments.ListPatchDeploymentsResponse()
            pb_resp = patch_deployments.ListPatchDeploymentsResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_list_patch_deployments(resp)
            return resp

    class _ListPatchJobInstanceDetails(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("ListPatchJobInstanceDetails")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_jobs.ListPatchJobInstanceDetailsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_jobs.ListPatchJobInstanceDetailsResponse:
            r"""Call the list patch job instance
            details method over HTTP.

                Args:
                    request (~.patch_jobs.ListPatchJobInstanceDetailsRequest):
                        The request object. Request to list details for all
                    instances that are part of a patch job.
                    retry (google.api_core.retry.Retry): Designation of what errors, if any,
                        should be retried.
                    timeout (float): The timeout for this request.
                    metadata (Sequence[Tuple[str, str]]): Strings which should be
                        sent along with the request as metadata.

                Returns:
                    ~.patch_jobs.ListPatchJobInstanceDetailsResponse:
                        A response message for listing the
                    instances details for a patch job.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1/{parent=projects/*/patchJobs/*}/instanceDetails",
                },
            ]
            request, metadata = self._interceptor.pre_list_patch_job_instance_details(
                request, metadata
            )
            pb_request = patch_jobs.ListPatchJobInstanceDetailsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_jobs.ListPatchJobInstanceDetailsResponse()
            pb_resp = patch_jobs.ListPatchJobInstanceDetailsResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_list_patch_job_instance_details(resp)
            return resp

    class _ListPatchJobs(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("ListPatchJobs")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_jobs.ListPatchJobsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_jobs.ListPatchJobsResponse:
            r"""Call the list patch jobs method over HTTP.

            Args:
                request (~.patch_jobs.ListPatchJobsRequest):
                    The request object. A request message for listing patch
                jobs.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.patch_jobs.ListPatchJobsResponse:
                    A response message for listing patch
                jobs.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "get",
                    "uri": "/v1/{parent=projects/*}/patchJobs",
                },
            ]
            request, metadata = self._interceptor.pre_list_patch_jobs(request, metadata)
            pb_request = patch_jobs.ListPatchJobsRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_jobs.ListPatchJobsResponse()
            pb_resp = patch_jobs.ListPatchJobsResponse.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_list_patch_jobs(resp)
            return resp

    class _PausePatchDeployment(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("PausePatchDeployment")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_deployments.PausePatchDeploymentRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_deployments.PatchDeployment:
            r"""Call the pause patch deployment method over HTTP.

            Args:
                request (~.patch_deployments.PausePatchDeploymentRequest):
                    The request object. A request message for pausing a patch
                deployment.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.patch_deployments.PatchDeployment:
                    Patch deployments are configurations that individual
                patch jobs use to complete a patch. These configurations
                include instance filter, package repository settings,
                and a schedule. For more information about creating and
                managing patch deployments, see `Scheduling patch
                jobs <https://cloud.google.com/compute/docs/os-patch-management/schedule-patch-jobs>`__.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1/{name=projects/*/patchDeployments/*}:pause",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_pause_patch_deployment(
                request, metadata
            )
            pb_request = patch_deployments.PausePatchDeploymentRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_deployments.PatchDeployment()
            pb_resp = patch_deployments.PatchDeployment.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_pause_patch_deployment(resp)
            return resp

    class _ResumePatchDeployment(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("ResumePatchDeployment")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_deployments.ResumePatchDeploymentRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_deployments.PatchDeployment:
            r"""Call the resume patch deployment method over HTTP.

            Args:
                request (~.patch_deployments.ResumePatchDeploymentRequest):
                    The request object. A request message for resuming a
                patch deployment.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.patch_deployments.PatchDeployment:
                    Patch deployments are configurations that individual
                patch jobs use to complete a patch. These configurations
                include instance filter, package repository settings,
                and a schedule. For more information about creating and
                managing patch deployments, see `Scheduling patch
                jobs <https://cloud.google.com/compute/docs/os-patch-management/schedule-patch-jobs>`__.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "post",
                    "uri": "/v1/{name=projects/*/patchDeployments/*}:resume",
                    "body": "*",
                },
            ]
            request, metadata = self._interceptor.pre_resume_patch_deployment(
                request, metadata
            )
            pb_request = patch_deployments.ResumePatchDeploymentRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_deployments.PatchDeployment()
            pb_resp = patch_deployments.PatchDeployment.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_resume_patch_deployment(resp)
            return resp

    class _UpdatePatchDeployment(OsConfigServiceRestStub):
        def __hash__(self):
            return hash("UpdatePatchDeployment")

        __REQUIRED_FIELDS_DEFAULT_VALUES: Dict[str, Any] = {}

        @classmethod
        def _get_unset_required_fields(cls, message_dict):
            return {
                k: v
                for k, v in cls.__REQUIRED_FIELDS_DEFAULT_VALUES.items()
                if k not in message_dict
            }

        def __call__(
            self,
            request: patch_deployments.UpdatePatchDeploymentRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, str]] = (),
        ) -> patch_deployments.PatchDeployment:
            r"""Call the update patch deployment method over HTTP.

            Args:
                request (~.patch_deployments.UpdatePatchDeploymentRequest):
                    The request object. A request message for updating a
                patch deployment.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, str]]): Strings which should be
                    sent along with the request as metadata.

            Returns:
                ~.patch_deployments.PatchDeployment:
                    Patch deployments are configurations that individual
                patch jobs use to complete a patch. These configurations
                include instance filter, package repository settings,
                and a schedule. For more information about creating and
                managing patch deployments, see `Scheduling patch
                jobs <https://cloud.google.com/compute/docs/os-patch-management/schedule-patch-jobs>`__.

            """

            http_options: List[Dict[str, str]] = [
                {
                    "method": "patch",
                    "uri": "/v1/{patch_deployment.name=projects/*/patchDeployments/*}",
                    "body": "patch_deployment",
                },
            ]
            request, metadata = self._interceptor.pre_update_patch_deployment(
                request, metadata
            )
            pb_request = patch_deployments.UpdatePatchDeploymentRequest.pb(request)
            transcoded_request = path_template.transcode(http_options, pb_request)

            # Jsonify the request body

            body = json_format.MessageToJson(
                transcoded_request["body"],
                including_default_value_fields=False,
                use_integers_for_enums=True,
            )
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]

            # Jsonify the query params
            query_params = json.loads(
                json_format.MessageToJson(
                    transcoded_request["query_params"],
                    including_default_value_fields=False,
                    use_integers_for_enums=True,
                )
            )
            query_params.update(self._get_unset_required_fields(query_params))

            query_params["$alt"] = "json;enum-encoding=int"

            # Send the request
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(self._session, method)(
                "{host}{uri}".format(host=self._host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
                data=body,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = patch_deployments.PatchDeployment()
            pb_resp = patch_deployments.PatchDeployment.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)
            resp = self._interceptor.post_update_patch_deployment(resp)
            return resp

    @property
    def cancel_patch_job(
        self,
    ) -> Callable[[patch_jobs.CancelPatchJobRequest], patch_jobs.PatchJob]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._CancelPatchJob(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def create_patch_deployment(
        self,
    ) -> Callable[
        [patch_deployments.CreatePatchDeploymentRequest],
        patch_deployments.PatchDeployment,
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._CreatePatchDeployment(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def delete_patch_deployment(
        self,
    ) -> Callable[[patch_deployments.DeletePatchDeploymentRequest], empty_pb2.Empty]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._DeletePatchDeployment(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def execute_patch_job(
        self,
    ) -> Callable[[patch_jobs.ExecutePatchJobRequest], patch_jobs.PatchJob]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ExecutePatchJob(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def get_patch_deployment(
        self,
    ) -> Callable[
        [patch_deployments.GetPatchDeploymentRequest], patch_deployments.PatchDeployment
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetPatchDeployment(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def get_patch_job(
        self,
    ) -> Callable[[patch_jobs.GetPatchJobRequest], patch_jobs.PatchJob]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetPatchJob(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def list_patch_deployments(
        self,
    ) -> Callable[
        [patch_deployments.ListPatchDeploymentsRequest],
        patch_deployments.ListPatchDeploymentsResponse,
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ListPatchDeployments(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def list_patch_job_instance_details(
        self,
    ) -> Callable[
        [patch_jobs.ListPatchJobInstanceDetailsRequest],
        patch_jobs.ListPatchJobInstanceDetailsResponse,
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ListPatchJobInstanceDetails(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def list_patch_jobs(
        self,
    ) -> Callable[[patch_jobs.ListPatchJobsRequest], patch_jobs.ListPatchJobsResponse]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ListPatchJobs(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def pause_patch_deployment(
        self,
    ) -> Callable[
        [patch_deployments.PausePatchDeploymentRequest],
        patch_deployments.PatchDeployment,
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._PausePatchDeployment(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def resume_patch_deployment(
        self,
    ) -> Callable[
        [patch_deployments.ResumePatchDeploymentRequest],
        patch_deployments.PatchDeployment,
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._ResumePatchDeployment(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def update_patch_deployment(
        self,
    ) -> Callable[
        [patch_deployments.UpdatePatchDeploymentRequest],
        patch_deployments.PatchDeployment,
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._UpdatePatchDeployment(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__ = ("OsConfigServiceRestTransport",)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
from google.type import date_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.osconfig.v1alpha",
    manifest={
        "InventoryView",
        "Inventory",
        "GetInventoryRequest",
        "ListInventoriesRequest",
        "ListInventoriesResponse",
    },
)


class InventoryView(proto.Enum):
    r"""The view for inventory objects.

    Values:
        INVENTORY_VIEW_UNSPECIFIED (0):
            The default value.
            The API defaults to the BASIC view.
        BASIC (1):
            Returns the basic inventory information that includes
            ``os_info``.
        FULL (2):
            Returns all fields.
    """
    INVENTORY_VIEW_UNSPECIFIED = 0
    BASIC = 1
    FULL = 2


class Inventory(proto.Message):
    r"""This API resource represents the available inventory data for a
    Compute Engine virtual machine (VM) instance at a given point in
    time.

    You can use this API resource to determine the inventory data of
    your VM.

    For more information, see `Information provided by OS inventory
    management <https://cloud.google.com/compute/docs/instances/os-inventory-management#data-collected>`__.

    Attributes:
        name (str):
            Output only. The ``Inventory`` API resource name.

            Format:
            ``projects/{project_number}/locations/{location}/instances/{instance_id}/inventory``
        os_info (google.cloud.osconfig_v1alpha.types.Inventory.OsInfo):
            Output only. Base level operating system
            information for the VM.
        items (MutableMapping[str, google.cloud.osconfig_v1alpha.types.Inventory.Item]):
            Output only. Inventory items related to the
            VM keyed by an opaque unique identifier for each
            inventory item. The identifier is unique to each
            distinct and addressable inventory item and will
            change, when there is a new package version.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp of the last reported
            inventory for the VM.
    """

    class OsInfo(proto.Message):
        r"""Operating system information for the VM.

        Attributes:
            hostname (str):
                The VM hostname.
            long_name (str):
                The operating system long name.
                For example 'Debian GNU/Linux 9' or 'Microsoft
                Window Server 2019 Datacenter'.
            short_name (str):
                The operating system short name.
                For example, 'windows' or 'debian'.
            version (str):
                The version of the operating system.
            architecture (str):
                The system architecture of the operating
                system.
            kernel_version (str):
                The kernel version of the operating system.
            kernel_release (str):
                The kernel release of the operating system.
            osconfig_agent_version (str):
                The current version of the OS Config agent
                running on the VM.
        """

        hostname: str = proto.Field(
            proto.STRING,
            number=9,
        )
        long_name: str = proto.Field(
            proto.STRING,
            number=2,
        )
        short_name: str = proto.Field(
            proto.STRING,
            number=3,
        )
        version: str = proto.Field(
            proto.STRING,
            number=4,
        )
        architecture: str = proto.Field(
            proto.STRING,
            number=5,
        )
        kernel_version: str = proto.Field(
            proto.STRING,
            number=6,
        )
        kernel_release: str = proto.Field(
            proto.STRING,
            number=7,
        )
        osconfig_agent_version: str = proto.Field(
            proto.STRING,
            number=8,
        )

    class Item(proto.Message):
        r"""A single piece of inventory on a VM.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            id (str):
                Identifier for this item, unique across items
                for this VM.
            origin_type (google.cloud.osconfig_v1alpha.types.Inventory.Item.OriginType):
                The origin of this inventory item.
            create_time (google.protobuf.timestamp_pb2.Timestamp):
                When this inventory item was first detected.
            update_time (google.protobuf.timestamp_pb2.Timestamp):
                When this inventory item was last modified.
            type_ (google.cloud.osconfig_v1alpha.types.Inventory.Item.Type):
                The specific type of inventory, correlating
                to its specific details.
            installed_package (google.cloud.osconfig_v1alpha.types.Inventory.SoftwarePackage):
                Software package present on the VM instance.

                This field is a member of `oneof`_ ``details``.
            available_package (google.cloud.osconfig_v1alpha.types.Inventory.SoftwarePackage):
                Software package available to be installed on
                the VM instance.

                This field is a member of `oneof`_ ``details``.
        """

        class OriginType(proto.Enum):
            r"""The origin of a specific inventory item.

            Values:
                ORIGIN_TYPE_UNSPECIFIED (0):
                    Invalid. An origin type must be specified.
                INVENTORY_REPORT (1):
                    This inventory item was discovered as the
                    result of the agent reporting inventory via the
                    reporting API.
            """
            ORIGIN_TYPE_UNSPECIFIED = 0
            INVENTORY_REPORT = 1

        class Type(proto.Enum):
            r"""The different types of inventory that are tracked on a VM.

            Values:
                TYPE_UNSPECIFIED (0):
                    Invalid. An type must be specified.
                INSTALLED_PACKAGE (1):
                    This represents a package that is installed
                    on the VM.
                AVAILABLE_PACKAGE (2):
                    This represents an update that is available
                    for a package.
            """
            TYPE_UNSPECIFIED = 0
            INSTALLED_PACKAGE = 1
            AVAILABLE_PACKAGE = 2

        id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        origin_type: "Inventory.Item.OriginType" = proto.Field(
            proto.ENUM,
            number=2,
            enum="Inventory.Item.OriginType",
        )
        create_time: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=8,
            message=timestamp_pb2.Timestamp,
        )
        update_time: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=9,
            message=timestamp_pb2.Timestamp,
        )
        type_: "Inventory.Item.Type" = proto.Field(
            proto.ENUM,
            number=5,
            enum="Inventory.Item.Type",
        )
        installed_package: "Inventory.SoftwarePackage" = proto.Field(
            proto.MESSAGE,
            number=6,
            oneof="details",
            message="Inventory.SoftwarePackage",
        )
        available_package: "Inventory.SoftwarePackage" = proto.Field(
            proto.MESSAGE,
            number=7,
            oneof="details",
            message="Inventory.SoftwarePackage",
        )

    class SoftwarePackage(proto.Message):
        r"""Software package information of the operating system.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            yum_package (google.cloud.osconfig_v1alpha.types.Inventory.VersionedPackage):
                Yum package info. For details about the yum package manager,
                see
                https://access.redhat.com/documentation/en-us/red_hat_enterprise_linux/6/html/deployment_guide/ch-yum.

                This field is a member of `oneof`_ ``details``.
            apt_package (google.cloud.osconfig_v1alpha.types.Inventory.VersionedPackage):
                Details of an APT package.
                For details about the apt package manager, see
                https://wiki.debian.org/Apt.

                This field is a member of `oneof`_ ``details``.
            zypper_package (google.cloud.osconfig_v1alpha.types.Inventory.VersionedPackage):
                Details of a Zypper package. For details about the Zypper
                package manager, see
                https://en.opensuse.org/SDB:Zypper_manual.

                This field is a member of `oneof`_ ``details``.
            googet_package (google.cloud.osconfig_v1alpha.types.Inventory.VersionedPackage):
                Details of a Googet package.
                For details about the googet package manager,
                see  https://github.com/google/googet.

                This field is a member of `oneof`_ ``details``.
            zypper_patch (google.cloud.osconfig_v1alpha.types.Inventory.ZypperPatch):
                Details of a Zypper patch. For details about the Zypper
                package manager, see
                https://en.opensuse.org/SDB:Zypper_manual.

                This field is a member of `oneof`_ ``details``.
            wua_package (google.cloud.osconfig_v1alpha.types.Inventory.WindowsUpdatePackage):
                Details of a Windows Update package. See
                https://docs.microsoft.com/en-us/windows/win32/api/_wua/ for
                information about Windows Update.

                This field is a member of `oneof`_ ``details``.
            qfe_package (google.cloud.osconfig_v1alpha.types.Inventory.WindowsQuickFixEngineeringPackage):
                Details of a Windows Quick Fix engineering
                package. See
                https://docs.microsoft.com/en-us/windows/win32/cimwin32prov/win32-quickfixengineering
                for info in Windows Quick Fix Engineering.

                This field is a member of `oneof`_ ``details``.
            cos_package (google.cloud.osconfig_v1alpha.types.Inventory.VersionedPackage):
                Details of a COS package.

                This field is a member of `oneof`_ ``details``.
            windows_application (google.cloud.osconfig_v1alpha.types.Inventory.WindowsApplication):
                Details of Windows Application.

                This field is a member of `oneof`_ ``details``.
        """

        yum_package: "Inventory.VersionedPackage" = proto.Field(
            proto.MESSAGE,
            number=1,
            oneof="details",
            message="Inventory.VersionedPackage",
        )
        apt_package: "Inventory.VersionedPackage" = proto.Field(
            proto.MESSAGE,
            number=2,
            oneof="details",
            message="Inventory.VersionedPackage",
        )
        zypper_package: "Inventory.VersionedPackage" = proto.Field(
            proto.MESSAGE,
            number=3,
            oneof="details",
            message="Inventory.VersionedPackage",
        )
        googet_package: "Inventory.VersionedPackage" = proto.Field(
            proto.MESSAGE,
            number=4,
            oneof="details",
            message="Inventory.VersionedPackage",
        )
        zypper_patch: "Inventory.ZypperPatch" = proto.Field(
            proto.MESSAGE,
            number=5,
            oneof="details",
            message="Inventory.ZypperPatch",
        )
        wua_package: "Inventory.WindowsUpdatePackage" = proto.Field(
            proto.MESSAGE,
            number=6,
            oneof="details",
            message="Inventory.WindowsUpdatePackage",
        )
        qfe_package: "Inventory.WindowsQuickFixEngineeringPackage" = proto.Field(
            proto.MESSAGE,
            number=7,
            oneof="details",
            message="Inventory.WindowsQuickFixEngineeringPackage",
        )
        cos_package: "Inventory.VersionedPackage" = proto.Field(
            proto.MESSAGE,
            number=8,
            oneof="details",
            message="Inventory.VersionedPackage",
        )
        windows_application: "Inventory.WindowsApplication" = proto.Field(
            proto.MESSAGE,
            number=9,
            oneof="details",
            message="Inventory.WindowsApplication",
        )

    class VersionedPackage(proto.Message):
        r"""Information related to the a standard versioned package.
        This includes package info for APT, Yum, Zypper, and Googet
        package managers.

        Attributes:
            package_name (str):
                The name of the package.
            architecture (str):
                The system architecture this package is
                intended for.
            version (str):
                The version of the package.
        """

        package_name: str = proto.Field(
            proto.STRING,
            number=4,
        )
        architecture: str = proto.Field(
            proto.STRING,
            number=2,
        )
        version: str = proto.Field(
            proto.STRING,
            number=3,
        )

    class ZypperPatch(proto.Message):
        r"""Details related to a Zypper Patch.

        Attributes:
            patch_name (str):
                The name of the patch.
            category (str):
                The category of the patch.
            severity (str):
                The severity specified for this patch
            summary (str):
                Any summary information provided about this
                patch.
        """

        patch_name: str = proto.Field(
            proto.STRING,
            number=5,
        )
        category: str = proto.Field(
            proto.STRING,
            number=2,
        )
        severity: str = proto.Field(
            proto.STRING,
            number=3,
        )
        summary: str = proto.Field(
            proto.STRING,
            number=4,
        )

    class WindowsUpdatePackage(proto.Message):
        r"""Details related to a Windows Update package. Field data and names
        are taken from Windows Update API IUpdate Interface:
        https://docs.microsoft.com/en-us/windows/win32/api/_wua/ Descriptive
        fields like title, and description are localized based on the locale
        of the VM being updated.

        Attributes:
            title (str):
                The localized title of the update package.
            description (str):
                The localized description of the update
                package.
            categories (MutableSequence[google.cloud.osconfig_v1alpha.types.Inventory.WindowsUpdatePackage.WindowsUpdateCategory]):
                The categories that are associated with this
                update package.
            kb_article_ids (MutableSequence[str]):
                A collection of Microsoft Knowledge Base
                article IDs that are associated with the update
                package.
            support_url (str):
                A hyperlink to the language-specific support
                information for the update.
            more_info_urls (MutableSequence[str]):
                A collection of URLs that provide more
                information about the update package.
            update_id (str):
                Gets the identifier of an update package.
                Stays the same across revisions.
            revision_number (int):
                The revision number of this update package.
            last_deployment_change_time (google.protobuf.timestamp_pb2.Timestamp):
                The last published date of the update, in
                (UTC) date and time.
        """

        class WindowsUpdateCategory(proto.Message):
            r"""Categories specified by the Windows Update.

            Attributes:
                id (str):
                    The identifier of the windows update
                    category.
                name (str):
                    The name of the windows update category.
            """

            id: str = proto.Field(
                proto.STRING,
                number=1,
            )
            name: str = proto.Field(
                proto.STRING,
                number=2,
            )

        title: str = proto.Field(
            proto.STRING,
            number=1,
        )
        description: str = proto.Field(
            proto.STRING,
            number=2,
        )
        categories: MutableSequence[
            "Inventory.WindowsUpdatePackage.WindowsUpdateCategory"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message="Inventory.WindowsUpdatePackage.WindowsUpdateCategory",
        )
        kb_article_ids: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=4,
        )
        support_url: str = proto.Field(
            proto.STRING,
            number=11,
        )
        more_info_urls: MutableSequence[str] = proto.RepeatedField(
            proto.STRING,
            number=5,
        )
        update_id: str = proto.Field(
            proto.STRING,
            number=6,
        )
        revision_number: int = proto.Field(
            proto.INT32,
            number=7,
        )
        last_deployment_change_time: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=10,
            message=timestamp_pb2.Timestamp,
        )

    class WindowsQuickFixEngineeringPackage(proto.Message):
        r"""Information related to a Quick Fix Engineering package.
        Fields are taken from Windows QuickFixEngineering Interface and
        match the source names:
        https://docs.microsoft.com/en-us/windows/win32/cimwin32prov/win32-quickfixengineering

        Attributes:
            caption (str):
                A short textual description of the QFE
                update.
            description (str):
                A textual description of the QFE update.
            hot_fix_id (str):
                Unique identifier associated with a
                particular QFE update.
            install_time (google.protobuf.timestamp_pb2.Timestamp):
                Date that the QFE update was installed. Mapped from
                installed_on field.
        """

        caption: str = proto.Field(
            proto.STRING,
            number=1,
        )
        description: str = proto.Field(
            proto.STRING,
            number=2,
        )
        hot_fix_id: str = proto.Field(
            proto.STRING,
            number=3,
        )
        install_time: timestamp_pb2.Timestamp = proto.Field(
            proto.MESSAGE,
            number=5,
            message=timestamp_pb2.Timestamp,
        )

    class WindowsApplication(proto.Message):
        r"""Contains information about a Windows application that is
        retrieved from the Windows Registry. For more information about
        these fields, see:
        https://docs.microsoft.com/en-us/windows/win32/msi/uninstall-registry-key

        Attributes:
            display_name (str):
                The name of the application or product.
            display_version (str):
                The version of the product or application in
                string format.
            publisher (str):
                The name of the manufacturer for the product
                or application.
            install_date (google.type.date_pb2.Date):
                The last time this product received service.
                The value of this property is replaced each time
                a patch is applied or removed from the product
                or the command-line option is used to repair the
                product.
            help_link (str):
                The internet address for technical support.
        """

        display_name: str = proto.Field(
            proto.STRING,
            number=1,
        )
        display_version: str = proto.Field(
            proto.STRING,
            number=2,
        )
        publisher: str = proto.Field(
            proto.STRING,
            number=3,
        )
        install_date: date_pb2.Date = proto.Field(
            proto.MESSAGE,
            number=4,
            message=date_pb2.Date,
        )
        help_link: str = proto.Field(
            proto.STRING,
            number=5,
        )

    name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    os_info: OsInfo = proto.Field(
        proto.MESSAGE,
        number=1,
        message=OsInfo,
    )
    items: MutableMapping[str, Item] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=2,
        message=Item,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )


class GetInventoryRequest(proto.Message):
    r"""A request message for getting inventory data for the
    specified VM.

    Attributes:
        name (str):
            Required. API resource name for inventory resource.

            Format:
            ``projects/{project}/locations/{location}/instances/{instance}/inventory``

            For ``{project}``, either ``project-number`` or
            ``project-id`` can be provided. For ``{instance}``, either
            Compute Engine ``instance-id`` or ``instance-name`` can be
            provided.
        view (google.cloud.osconfig_v1alpha.types.InventoryView):
            Inventory view indicating what information
            should be included in the inventory resource. If
            unspecified, the default view is BASIC.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    view: "InventoryView" = proto.Field(
        proto.ENUM,
        number=2,
        enum="InventoryView",
    )


class ListInventoriesRequest(proto.Message):
    r"""A request message for listing inventory data for all VMs in
    the specified location.

    Attributes:
        parent (str):
            Required. The parent resource name.

            Format:
            ``projects/{project}/locations/{location}/instances/-``

            For ``{project}``, either ``project-number`` or
            ``project-id`` can be provided.
        view (google.cloud.osconfig_v1alpha.types.InventoryView):
            Inventory view indicating what information
            should be included in the inventory resource. If
            unspecified, the default view is BASIC.
        page_size (int):
            The maximum number of results to return.
        page_token (str):
            A pagination token returned from a previous call to
            ``ListInventories`` that indicates where this listing should
            continue from.
        filter (str):
            If provided, this field specifies the criteria that must be
            met by a ``Inventory`` API resource to be included in the
            response.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    view: "InventoryView" = proto.Field(
        proto.ENUM,
        number=2,
        enum="InventoryView",
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=4,
    )
    filter: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListInventoriesResponse(proto.Message):
    r"""A response message for listing inventory data for all VMs in
    a specified location.

    Attributes:
        inventories (MutableSequence[google.cloud.osconfig_v1alpha.types.Inventory]):
            List of inventory objects.
        next_page_token (str):
            The pagination token to retrieve the next
            page of inventory objects.
    """

    @property
    def raw_page(self):
        return self

    inventories: MutableSequence["Inventory"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="Inventory",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import uuid

from google.protobuf import timestamp_pb2 as timestamp  # type: ignore

from google.cloud.osconfig_v1.services.os_config_service import OsConfigServiceClient
from google.cloud.osconfig_v1.types import patch_deployments, patch_jobs


class TestOsConfigServiceV1(object):
    def test_patch_job(self):
        project_id = os.environ["PROJECT_ID"]
        client = OsConfigServiceClient()

        # ExecutePatchJob
        request = patch_jobs.ExecutePatchJobRequest(
            parent=f"projects/{project_id}",
            description="Python Client Library System Test",
            instance_filter=patch_jobs.PatchInstanceFilter(all_=True),
        )
        patch_job = client.execute_patch_job(request)
        assert patch_job is not None

        # GetPatchJob
        request = patch_jobs.GetPatchJobRequest(name=patch_job.name)
        patch_job = client.get_patch_job(request)
        assert patch_job.description == "Python Client Library System Test"

        # ListPatchJobInstanceDetails
        request = patch_jobs.ListPatchJobInstanceDetailsRequest(parent=patch_job.name)
        response = client.list_patch_job_instance_details(request)
        assert len(response.patch_job_instance_details) >= 0

        # CancelPatchJob
        request = patch_jobs.CancelPatchJobRequest(name=patch_job.name)
        patch_job = client.cancel_patch_job(request)
        assert patch_job.state == patch_jobs.PatchJob.State.CANCELED

        # ListPatchJobs
        request = patch_jobs.ListPatchJobsRequest(parent=f"projects/{project_id}")
        response = client.list_patch_jobs(request)
        assert response.patch_jobs

    def test_patch_deployment(self):
        project_id = os.environ["PROJECT_ID"]
        client = OsConfigServiceClient()

        patch_deployment = patch_deployments.PatchDeployment(
            instance_filter=patch_jobs.PatchInstanceFilter(all_=True),
            one_time_schedule=patch_deployments.OneTimeSchedule(
                execute_time=timestamp.Timestamp(seconds=200000000000)
            ),
        )
        patch_deployment_id = "python-client-library-test-" + str(uuid.uuid1())
        patch_deployment_name = (
            f"projects/{project_id}/patchDeployments/{patch_deployment_id}"
        )

        # CreatePatchDeploymentRequest
        request = patch_deployments.CreatePatchDeploymentRequest(
            parent=f"projects/{project_id}",
            patch_deployment_id=patch_deployment_id,
            patch_deployment=patch_deployment,
        )
        patch_deployment = client.create_patch_deployment(request)
        assert patch_deployment_id in patch_deployment.name

        # GetPatchDeployment
        request = patch_deployments.GetPatchDeploymentRequest(
            name=patch_deployment_name
        )
        patch_deployment = client.get_patch_deployment(request)
        assert patch_deployment_id in patch_deployment.name

        # ListPatchDeployments
        request = patch_deployments.ListPatchDeploymentsRequest(
            parent=f"projects/{project_id}"
        )
        response = client.list_patch_deployments(request)
        assert response.patch_deployments

        # DeletePatchDeployment
        request = patch_deployments.DeletePatchDeploymentRequest(
            name=patch_deployment_name
        )
        client.delete_patch_deployment(request)



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.osconfig.v1",
    manifest={
        "OSPolicy",
    },
)


class OSPolicy(proto.Message):
    r"""An OS policy defines the desired state configuration for a
    VM.

    Attributes:
        id (str):
            Required. The id of the OS policy with the following
            restrictions:

            -  Must contain only lowercase letters, numbers, and
               hyphens.
            -  Must start with a letter.
            -  Must be between 1-63 characters.
            -  Must end with a number or a letter.
            -  Must be unique within the assignment.
        description (str):
            Policy description.
            Length of the description is limited to 1024
            characters.
        mode (google.cloud.osconfig_v1.types.OSPolicy.Mode):
            Required. Policy mode
        resource_groups (MutableSequence[google.cloud.osconfig_v1.types.OSPolicy.ResourceGroup]):
            Required. List of resource groups for the policy. For a
            particular VM, resource groups are evaluated in the order
            specified and the first resource group that is applicable is
            selected and the rest are ignored.

            If none of the resource groups are applicable for a VM, the
            VM is considered to be non-compliant w.r.t this policy. This
            behavior can be toggled by the flag
            ``allow_no_resource_group_match``
        allow_no_resource_group_match (bool):
            This flag determines the OS policy compliance status when
            none of the resource groups within the policy are applicable
            for a VM. Set this value to ``true`` if the policy needs to
            be reported as compliant even if the policy has nothing to
            validate or enforce.
    """

    class Mode(proto.Enum):
        r"""Policy mode

        Values:
            MODE_UNSPECIFIED (0):
                Invalid mode
            VALIDATION (1):
                This mode checks if the configuration
                resources in the policy are in their desired
                state. No actions are performed if they are not
                in the desired state. This mode is used for
                reporting purposes.
            ENFORCEMENT (2):
                This mode checks if the configuration
                resources in the policy are in their desired
                state, and if not, enforces the desired state.
        """
        MODE_UNSPECIFIED = 0
        VALIDATION = 1
        ENFORCEMENT = 2

    class InventoryFilter(proto.Message):
        r"""Filtering criteria to select VMs based on inventory details.

        Attributes:
            os_short_name (str):
                Required. The OS short name
            os_version (str):
                The OS version

                Prefix matches are supported if asterisk(*) is provided as
                the last character. For example, to match all versions with
                a major version of ``7``, specify the following value for
                this field ``7.*``

                An empty string matches all OS versions.
        """

        os_short_name: str = proto.Field(
            proto.STRING,
            number=1,
        )
        os_version: str = proto.Field(
            proto.STRING,
            number=2,
        )

    class Resource(proto.Message):
        r"""An OS policy resource is used to define the desired state
        configuration and provides a specific functionality like
        installing/removing packages, executing a script etc.

        The system ensures that resources are always in their desired
        state by taking necessary actions if they have drifted from
        their desired state.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            id (str):
                Required. The id of the resource with the following
                restrictions:

                -  Must contain only lowercase letters, numbers, and
                   hyphens.
                -  Must start with a letter.
                -  Must be between 1-63 characters.
                -  Must end with a number or a letter.
                -  Must be unique within the OS policy.
            pkg (google.cloud.osconfig_v1.types.OSPolicy.Resource.PackageResource):
                Package resource

                This field is a member of `oneof`_ ``resource_type``.
            repository (google.cloud.osconfig_v1.types.OSPolicy.Resource.RepositoryResource):
                Package repository resource

                This field is a member of `oneof`_ ``resource_type``.
            exec_ (google.cloud.osconfig_v1.types.OSPolicy.Resource.ExecResource):
                Exec resource

                This field is a member of `oneof`_ ``resource_type``.
            file (google.cloud.osconfig_v1.types.OSPolicy.Resource.FileResource):
                File resource

                This field is a member of `oneof`_ ``resource_type``.
        """

        class File(proto.Message):
            r"""A remote or local file.

            This message has `oneof`_ fields (mutually exclusive fields).
            For each oneof, at most one member field can be set at the same time.
            Setting any member of the oneof automatically clears all other
            members.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                remote (google.cloud.osconfig_v1.types.OSPolicy.Resource.File.Remote):
                    A generic remote file.

                    This field is a member of `oneof`_ ``type``.
                gcs (google.cloud.osconfig_v1.types.OSPolicy.Resource.File.Gcs):
                    A Cloud Storage object.

                    This field is a member of `oneof`_ ``type``.
                local_path (str):
                    A local path within the VM to use.

                    This field is a member of `oneof`_ ``type``.
                allow_insecure (bool):
                    Defaults to false. When false, files are
                    subject to validations based on the file type:
                    Remote: A checksum must be specified.
                    Cloud Storage: An object generation number must
                    be specified.
            """

            class Remote(proto.Message):
                r"""Specifies a file available via some URI.

                Attributes:
                    uri (str):
                        Required. URI from which to fetch the object. It should
                        contain both the protocol and path following the format
                        ``{protocol}://{location}``.
                    sha256_checksum (str):
                        SHA256 checksum of the remote file.
                """

                uri: str = proto.Field(
                    proto.STRING,
                    number=1,
                )
                sha256_checksum: str = proto.Field(
                    proto.STRING,
                    number=2,
                )

            class Gcs(proto.Message):
                r"""Specifies a file available as a Cloud Storage Object.

                Attributes:
                    bucket (str):
                        Required. Bucket of the Cloud Storage object.
                    object_ (str):
                        Required. Name of the Cloud Storage object.
                    generation (int):
                        Generation number of the Cloud Storage
                        object.
                """

                bucket: str = proto.Field(
                    proto.STRING,
                    number=1,
                )
                object_: str = proto.Field(
                    proto.STRING,
                    number=2,
                )
                generation: int = proto.Field(
                    proto.INT64,
                    number=3,
                )

            remote: "OSPolicy.Resource.File.Remote" = proto.Field(
                proto.MESSAGE,
                number=1,
                oneof="type",
                message="OSPolicy.Resource.File.Remote",
            )
            gcs: "OSPolicy.Resource.File.Gcs" = proto.Field(
                proto.MESSAGE,
                number=2,
                oneof="type",
                message="OSPolicy.Resource.File.Gcs",
            )
            local_path: str = proto.Field(
                proto.STRING,
                number=3,
                oneof="type",
            )
            allow_insecure: bool = proto.Field(
                proto.BOOL,
                number=4,
            )

        class PackageResource(proto.Message):
            r"""A resource that manages a system package.

            This message has `oneof`_ fields (mutually exclusive fields).
            For each oneof, at most one member field can be set at the same time.
            Setting any member of the oneof automatically clears all other
            members.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                desired_state (google.cloud.osconfig_v1.types.OSPolicy.Resource.PackageResource.DesiredState):
                    Required. The desired state the agent should
                    maintain for this package.
                apt (google.cloud.osconfig_v1.types.OSPolicy.Resource.PackageResource.APT):
                    A package managed by Apt.

                    This field is a member of `oneof`_ ``system_package``.
                deb (google.cloud.osconfig_v1.types.OSPolicy.Resource.PackageResource.Deb):
                    A deb package file.

                    This field is a member of `oneof`_ ``system_package``.
                yum (google.cloud.osconfig_v1.types.OSPolicy.Resource.PackageResource.YUM):
                    A package managed by YUM.

                    This field is a member of `oneof`_ ``system_package``.
                zypper (google.cloud.osconfig_v1.types.OSPolicy.Resource.PackageResource.Zypper):
                    A package managed by Zypper.

                    This field is a member of `oneof`_ ``system_package``.
                rpm (google.cloud.osconfig_v1.types.OSPolicy.Resource.PackageResource.RPM):
                    An rpm package file.

                    This field is a member of `oneof`_ ``system_package``.
                googet (google.cloud.osconfig_v1.types.OSPolicy.Resource.PackageResource.GooGet):
                    A package managed by GooGet.

                    This field is a member of `oneof`_ ``system_package``.
                msi (google.cloud.osconfig_v1.types.OSPolicy.Resource.PackageResource.MSI):
                    An MSI package.

                    This field is a member of `oneof`_ ``system_package``.
            """

            class DesiredState(proto.Enum):
                r"""The desired state that the OS Config agent maintains on the
                VM.

                Values:
                    DESIRED_STATE_UNSPECIFIED (0):
                        Unspecified is invalid.
                    INSTALLED (1):
                        Ensure that the package is installed.
                    REMOVED (2):
                        The agent ensures that the package is not
                        installed and uninstalls it if detected.
                """
                DESIRED_STATE_UNSPECIFIED = 0
                INSTALLED = 1
                REMOVED = 2

            class Deb(proto.Message):
                r"""A deb package file. dpkg packages only support INSTALLED
                state.

                Attributes:
                    source (google.cloud.osconfig_v1.types.OSPolicy.Resource.File):
                        Required. A deb package.
                    pull_deps (bool):
                        Whether dependencies should also be installed.

                        -  install when false: ``dpkg -i package``
                        -  install when true:
                           ``apt-get update && apt-get -y install package.deb``
                """

                source: "OSPolicy.Resource.File" = proto.Field(
                    proto.MESSAGE,
                    number=1,
                    message="OSPolicy.Resource.File",
                )
                pull_deps: bool = proto.Field(
                    proto.BOOL,
                    number=2,
                )

            class APT(proto.Message):
                r"""A package managed by APT.

                -  install: ``apt-get update && apt-get -y install [name]``
                -  remove: ``apt-get -y remove [name]``

                Attributes:
                    name (str):
                        Required. Package name.
                """

                name: str = proto.Field(
                    proto.STRING,
                    number=1,
                )

            class RPM(proto.Message):
                r"""An RPM package file. RPM packages only support INSTALLED
                state.

                Attributes:
                    source (google.cloud.osconfig_v1.types.OSPolicy.Resource.File):
                        Required. An rpm package.
                    pull_deps (bool):
                        Whether dependencies should also be installed.

                        -  install when false:
                           ``rpm --upgrade --replacepkgs package.rpm``
                        -  install when true: ``yum -y install package.rpm`` or
                           ``zypper -y install package.rpm``
                """

                source: "OSPolicy.Resource.File" = proto.Field(
                    proto.MESSAGE,
                    number=1,
                    message="OSPolicy.Resource.File",
                )
                pull_deps: bool = proto.Field(
                    proto.BOOL,
                    number=2,
                )

            class YUM(proto.Message):
                r"""A package managed by YUM.

                -  install: ``yum -y install package``
                -  remove: ``yum -y remove package``

                Attributes:
                    name (str):
                        Required. Package name.
                """

                name: str = proto.Field(
                    proto.STRING,
                    number=1,
                )

            class Zypper(proto.Message):
                r"""A package managed by Zypper.

                -  install: ``zypper -y install package``
                -  remove: ``zypper -y rm package``

                Attributes:
                    name (str):
                        Required. Package name.
                """

                name: str = proto.Field(
                    proto.STRING,
                    number=1,
                )

            class GooGet(proto.Message):
                r"""A package managed by GooGet.

                -  install: ``googet -noconfirm install package``
                -  remove: ``googet -noconfirm remove package``

                Attributes:
                    name (str):
                        Required. Package name.
                """

                name: str = proto.Field(
                    proto.STRING,
                    number=1,
                )

            class MSI(proto.Message):
                r"""An MSI package. MSI packages only support INSTALLED state.

                Attributes:
                    source (google.cloud.osconfig_v1.types.OSPolicy.Resource.File):
                        Required. The MSI package.
                    properties (MutableSequence[str]):
                        Additional properties to use during installation. This
                        should be in the format of Property=Setting. Appended to the
                        defaults of ``ACTION=INSTALL REBOOT=ReallySuppress``.
                """

                source: "OSPolicy.Resource.File" = proto.Field(
                    proto.MESSAGE,
                    number=1,
                    message="OSPolicy.Resource.File",
                )
                properties: MutableSequence[str] = proto.RepeatedField(
                    proto.STRING,
                    number=2,
                )

            desired_state: "OSPolicy.Resource.PackageResource.DesiredState" = (
                proto.Field(
                    proto.ENUM,
                    number=1,
                    enum="OSPolicy.Resource.PackageResource.DesiredState",
                )
            )
            apt: "OSPolicy.Resource.PackageResource.APT" = proto.Field(
                proto.MESSAGE,
                number=2,
                oneof="system_package",
                message="OSPolicy.Resource.PackageResource.APT",
            )
            deb: "OSPolicy.Resource.PackageResource.Deb" = proto.Field(
                proto.MESSAGE,
                number=3,
                oneof="system_package",
                message="OSPolicy.Resource.PackageResource.Deb",
            )
            yum: "OSPolicy.Resource.PackageResource.YUM" = proto.Field(
                proto.MESSAGE,
                number=4,
                oneof="system_package",
                message="OSPolicy.Resource.PackageResource.YUM",
            )
            zypper: "OSPolicy.Resource.PackageResource.Zypper" = proto.Field(
                proto.MESSAGE,
                number=5,
                oneof="system_package",
                message="OSPolicy.Resource.PackageResource.Zypper",
            )
            rpm: "OSPolicy.Resource.PackageResource.RPM" = proto.Field(
                proto.MESSAGE,
                number=6,
                oneof="system_package",
                message="OSPolicy.Resource.PackageResource.RPM",
            )
            googet: "OSPolicy.Resource.PackageResource.GooGet" = proto.Field(
                proto.MESSAGE,
                number=7,
                oneof="system_package",
                message="OSPolicy.Resource.PackageResource.GooGet",
            )
            msi: "OSPolicy.Resource.PackageResource.MSI" = proto.Field(
                proto.MESSAGE,
                number=8,
                oneof="system_package",
                message="OSPolicy.Resource.PackageResource.MSI",
            )

        class RepositoryResource(proto.Message):
            r"""A resource that manages a package repository.

            This message has `oneof`_ fields (mutually exclusive fields).
            For each oneof, at most one member field can be set at the same time.
            Setting any member of the oneof automatically clears all other
            members.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                apt (google.cloud.osconfig_v1.types.OSPolicy.Resource.RepositoryResource.AptRepository):
                    An Apt Repository.

                    This field is a member of `oneof`_ ``repository``.
                yum (google.cloud.osconfig_v1.types.OSPolicy.Resource.RepositoryResource.YumRepository):
                    A Yum Repository.

                    This field is a member of `oneof`_ ``repository``.
                zypper (google.cloud.osconfig_v1.types.OSPolicy.Resource.RepositoryResource.ZypperRepository):
                    A Zypper Repository.

                    This field is a member of `oneof`_ ``repository``.
                goo (google.cloud.osconfig_v1.types.OSPolicy.Resource.RepositoryResource.GooRepository):
                    A Goo Repository.

                    This field is a member of `oneof`_ ``repository``.
            """

            class AptRepository(proto.Message):
                r"""Represents a single apt package repository. These will be added to a
                repo file that will be managed at
                ``/etc/apt/sources.list.d/google_osconfig.list``.

                Attributes:
                    archive_type (google.cloud.osconfig_v1.types.OSPolicy.Resource.RepositoryResource.AptRepository.ArchiveType):
                        Required. Type of archive files in this
                        repository.
                    uri (str):
                        Required. URI for this repository.
                    distribution (str):
                        Required. Distribution of this repository.
                    components (MutableSequence[str]):
                        Required. List of components for this
                        repository. Must contain at least one item.
                    gpg_key (str):
                        URI of the key file for this repository. The agent maintains
                        a keyring at
                        ``/etc/apt/trusted.gpg.d/osconfig_agent_managed.gpg``.
                """

                class ArchiveType(proto.Enum):
                    r"""Type of archive.

                    Values:
                        ARCHIVE_TYPE_UNSPECIFIED (0):
                            Unspecified is invalid.
                        DEB (1):
                            Deb indicates that the archive contains
                            binary files.
                        DEB_SRC (2):
                            Deb-src indicates that the archive contains
                            source files.
                    """
                    ARCHIVE_TYPE_UNSPECIFIED = 0
                    DEB = 1
                    DEB_SRC = 2

                archive_type: "OSPolicy.Resource.RepositoryResource.AptRepository.ArchiveType" = proto.Field(
                    proto.ENUM,
                    number=1,
                    enum="OSPolicy.Resource.RepositoryResource.AptRepository.ArchiveType",
                )
                uri: str = proto.Field(
                    proto.STRING,
                    number=2,
                )
                distribution: str = proto.Field(
                    proto.STRING,
                    number=3,
                )
                components: MutableSequence[str] = proto.RepeatedField(
                    proto.STRING,
                    number=4,
                )
                gpg_key: str = proto.Field(
                    proto.STRING,
                    number=5,
                )

            class YumRepository(proto.Message):
                r"""Represents a single yum package repository. These are added to a
                repo file that is managed at
                ``/etc/yum.repos.d/google_osconfig.repo``.

                Attributes:
                    id (str):
                        Required. A one word, unique name for this repository. This
                        is the ``repo id`` in the yum config file and also the
                        ``display_name`` if ``display_name`` is omitted. This id is
                        also used as the unique identifier when checking for
                        resource conflicts.
                    display_name (str):
                        The display name of the repository.
                    base_url (str):
                        Required. The location of the repository
                        directory.
                    gpg_keys (MutableSequence[str]):
                        URIs of GPG keys.
                """

                id: str = proto.Field(
                    proto.STRING,
                    number=1,
                )
                display_name: str = proto.Field(
                    proto.STRING,
                    number=2,
                )
                base_url: str = proto.Field(
                    proto.STRING,
                    number=3,
                )
                gpg_keys: MutableSequence[str] = proto.RepeatedField(
                    proto.STRING,
                    number=4,
                )

            class ZypperRepository(proto.Message):
                r"""Represents a single zypper package repository. These are added to a
                repo file that is managed at
                ``/etc/zypp/repos.d/google_osconfig.repo``.

                Attributes:
                    id (str):
                        Required. A one word, unique name for this repository. This
                        is the ``repo id`` in the zypper config file and also the
                        ``display_name`` if ``display_name`` is omitted. This id is
                        also used as the unique identifier when checking for
                        GuestPolicy conflicts.
                    display_name (str):
                        The display name of the repository.
                    base_url (str):
                        Required. The location of the repository
                        directory.
                    gpg_keys (MutableSequence[str]):
                        URIs of GPG keys.
                """

                id: str = proto.Field(
                    proto.STRING,
                    number=1,
                )
                display_name: str = proto.Field(
                    proto.STRING,
                    number=2,
                )
                base_url: str = proto.Field(
                    proto.STRING,
                    number=3,
                )
                gpg_keys: MutableSequence[str] = proto.RepeatedField(
                    proto.STRING,
                    number=4,
                )

            class GooRepository(proto.Message):
                r"""Represents a Goo package repository. These are added to a repo file
                that is managed at
                ``C:/ProgramData/GooGet/repos/google_osconfig.repo``.

                Attributes:
                    name (str):
                        Required. The name of the repository.
                    url (str):
                        Required. The url of the repository.
                """

                name: str = proto.Field(
                    proto.STRING,
                    number=1,
                )
                url: str = proto.Field(
                    proto.STRING,
                    number=2,
                )

            apt: "OSPolicy.Resource.RepositoryResource.AptRepository" = proto.Field(
                proto.MESSAGE,
                number=1,
                oneof="repository",
                message="OSPolicy.Resource.RepositoryResource.AptRepository",
            )
            yum: "OSPolicy.Resource.RepositoryResource.YumRepository" = proto.Field(
                proto.MESSAGE,
                number=2,
                oneof="repository",
                message="OSPolicy.Resource.RepositoryResource.YumRepository",
            )
            zypper: "OSPolicy.Resource.RepositoryResource.ZypperRepository" = (
                proto.Field(
                    proto.MESSAGE,
                    number=3,
                    oneof="repository",
                    message="OSPolicy.Resource.RepositoryResource.ZypperRepository",
                )
            )
            goo: "OSPolicy.Resource.RepositoryResource.GooRepository" = proto.Field(
                proto.MESSAGE,
                number=4,
                oneof="repository",
                message="OSPolicy.Resource.RepositoryResource.GooRepository",
            )

        class ExecResource(proto.Message):
            r"""A resource that allows executing scripts on the VM.

            The ``ExecResource`` has 2 stages: ``validate`` and ``enforce`` and
            both stages accept a script as an argument to execute.

            When the ``ExecResource`` is applied by the agent, it first executes
            the script in the ``validate`` stage. The ``validate`` stage can
            signal that the ``ExecResource`` is already in the desired state by
            returning an exit code of ``100``. If the ``ExecResource`` is not in
            the desired state, it should return an exit code of ``101``. Any
            other exit code returned by this stage is considered an error.

            If the ``ExecResource`` is not in the desired state based on the
            exit code from the ``validate`` stage, the agent proceeds to execute
            the script from the ``enforce`` stage. If the ``ExecResource`` is
            already in the desired state, the ``enforce`` stage will not be run.
            Similar to ``validate`` stage, the ``enforce`` stage should return
            an exit code of ``100`` to indicate that the resource in now in its
            desired state. Any other exit code is considered an error.

            NOTE: An exit code of ``100`` was chosen over ``0`` (and ``101`` vs
            ``1``) to have an explicit indicator of ``in desired state``,
            ``not in desired state`` and errors. Because, for example,
            Powershell will always return an exit code of ``0`` unless an
            ``exit`` statement is provided in the script. So, for reasons of
            consistency and being explicit, exit codes ``100`` and ``101`` were
            chosen.

            Attributes:
                validate (google.cloud.osconfig_v1.types.OSPolicy.Resource.ExecResource.Exec):
                    Required. What to run to validate this
                    resource is in the desired state. An exit code
                    of 100 indicates "in desired state", and exit
                    code of 101 indicates "not in desired state".
                    Any other exit code indicates a failure running
                    validate.
                enforce (google.cloud.osconfig_v1.types.OSPolicy.Resource.ExecResource.Exec):
                    What to run to bring this resource into the
                    desired state. An exit code of 100 indicates
                    "success", any other exit code indicates a
                    failure running enforce.
            """

            class Exec(proto.Message):
                r"""A file or script to execute.

                This message has `oneof`_ fields (mutually exclusive fields).
                For each oneof, at most one member field can be set at the same time.
                Setting any member of the oneof automatically clears all other
                members.

                .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

                Attributes:
                    file (google.cloud.osconfig_v1.types.OSPolicy.Resource.File):
                        A remote or local file.

                        This field is a member of `oneof`_ ``source``.
                    script (str):
                        An inline script.
                        The size of the script is limited to 1024
                        characters.

                        This field is a member of `oneof`_ ``source``.
                    args (MutableSequence[str]):
                        Optional arguments to pass to the source
                        during execution.
                    interpreter (google.cloud.osconfig_v1.types.OSPolicy.Resource.ExecResource.Exec.Interpreter):
                        Required. The script interpreter to use.
                    output_file_path (str):
                        Only recorded for enforce Exec.
                        Path to an output file (that is created by this
                        Exec) whose content will be recorded in
                        OSPolicyResourceCompliance after a successful
                        run. Absence or failure to read this file will
                        result in this ExecResource being non-compliant.
                        Output file size is limited to 100K bytes.
                """

                class Interpreter(proto.Enum):
                    r"""The interpreter to use.

                    Values:
                        INTERPRETER_UNSPECIFIED (0):
                            Invalid value, the request will return
                            validation error.
                        NONE (1):
                            If an interpreter is not specified, the source is executed
                            directly. This execution, without an interpreter, only
                            succeeds for executables and scripts that have shebang
                            lines.
                        SHELL (2):
                            Indicates that the script runs with ``/bin/sh`` on Linux and
                            ``cmd.exe`` on Windows.
                        POWERSHELL (3):
                            Indicates that the script runs with
                            PowerShell.
                    """
                    INTERPRETER_UNSPECIFIED = 0
                    NONE = 1
                    SHELL = 2
                    POWERSHELL = 3

                file: "OSPolicy.Resource.File" = proto.Field(
                    proto.MESSAGE,
                    number=1,
                    oneof="source",
                    message="OSPolicy.Resource.File",
                )
                script: str = proto.Field(
                    proto.STRING,
                    number=2,
                    oneof="source",
                )
                args: MutableSequence[str] = proto.RepeatedField(
                    proto.STRING,
                    number=3,
                )
                interpreter: "OSPolicy.Resource.ExecResource.Exec.Interpreter" = (
                    proto.Field(
                        proto.ENUM,
                        number=4,
                        enum="OSPolicy.Resource.ExecResource.Exec.Interpreter",
                    )
                )
                output_file_path: str = proto.Field(
                    proto.STRING,
                    number=5,
                )

            validate: "OSPolicy.Resource.ExecResource.Exec" = proto.Field(
                proto.MESSAGE,
                number=1,
                message="OSPolicy.Resource.ExecResource.Exec",
            )
            enforce: "OSPolicy.Resource.ExecResource.Exec" = proto.Field(
                proto.MESSAGE,
                number=2,
                message="OSPolicy.Resource.ExecResource.Exec",
            )

        class FileResource(proto.Message):
            r"""A resource that manages the state of a file.

            This message has `oneof`_ fields (mutually exclusive fields).
            For each oneof, at most one member field can be set at the same time.
            Setting any member of the oneof automatically clears all other
            members.

            .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

            Attributes:
                file (google.cloud.osconfig_v1.types.OSPolicy.Resource.File):
                    A remote or local source.

                    This field is a member of `oneof`_ ``source``.
                content (str):
                    A a file with this content.
                    The size of the content is limited to 1024
                    characters.

                    This field is a member of `oneof`_ ``source``.
                path (str):
                    Required. The absolute path of the file
                    within the VM.
                state (google.cloud.osconfig_v1.types.OSPolicy.Resource.FileResource.DesiredState):
                    Required. Desired state of the file.
                permissions (str):
                    Consists of three octal digits which
                    represent, in order, the permissions of the
                    owner, group, and other users for the file
                    (similarly to the numeric mode used in the linux
                    chmod utility). Each digit represents a three
                    bit number with the 4 bit corresponding to the
                    read permissions, the 2 bit corresponds to the
                    write bit, and the one bit corresponds to the
                    execute permission. Default behavior is 755.

                    Below are some examples of permissions and their
                    associated values: read, write, and execute: 7
                    read and execute: 5
                    read and write: 6
                    read only: 4
            """

            class DesiredState(proto.Enum):
                r"""Desired state of the file.

                Values:
                    DESIRED_STATE_UNSPECIFIED (0):
                        Unspecified is invalid.
                    PRESENT (1):
                        Ensure file at path is present.
                    ABSENT (2):
                        Ensure file at path is absent.
                    CONTENTS_MATCH (3):
                        Ensure the contents of the file at path
                        matches. If the file does not exist it will be
                        created.
                """
                DESIRED_STATE_UNSPECIFIED = 0
                PRESENT = 1
                ABSENT = 2
                CONTENTS_MATCH = 3

            file: "OSPolicy.Resource.File" = proto.Field(
                proto.MESSAGE,
                number=1,
                oneof="source",
                message="OSPolicy.Resource.File",
            )
            content: str = proto.Field(
                proto.STRING,
                number=2,
                oneof="source",
            )
            path: str = proto.Field(
                proto.STRING,
                number=3,
            )
            state: "OSPolicy.Resource.FileResource.DesiredState" = proto.Field(
                proto.ENUM,
                number=4,
                enum="OSPolicy.Resource.FileResource.DesiredState",
            )
            permissions: str = proto.Field(
                proto.STRING,
                number=5,
            )

        id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        pkg: "OSPolicy.Resource.PackageResource" = proto.Field(
            proto.MESSAGE,
            number=2,
            oneof="resource_type",
            message="OSPolicy.Resource.PackageResource",
        )
        repository: "OSPolicy.Resource.RepositoryResource" = proto.Field(
            proto.MESSAGE,
            number=3,
            oneof="resource_type",
            message="OSPolicy.Resource.RepositoryResource",
        )
        exec_: "OSPolicy.Resource.ExecResource" = proto.Field(
            proto.MESSAGE,
            number=4,
            oneof="resource_type",
            message="OSPolicy.Resource.ExecResource",
        )
        file: "OSPolicy.Resource.FileResource" = proto.Field(
            proto.MESSAGE,
            number=5,
            oneof="resource_type",
            message="OSPolicy.Resource.FileResource",
        )

    class ResourceGroup(proto.Message):
        r"""Resource groups provide a mechanism to group OS policy resources.

        Resource groups enable OS policy authors to create a single OS
        policy to be applied to VMs running different operating Systems.

        When the OS policy is applied to a target VM, the appropriate
        resource group within the OS policy is selected based on the
        ``OSFilter`` specified within the resource group.

        Attributes:
            inventory_filters (MutableSequence[google.cloud.osconfig_v1.types.OSPolicy.InventoryFilter]):
                List of inventory filters for the resource group.

                The resources in this resource group are applied to the
                target VM if it satisfies at least one of the following
                inventory filters.

                For example, to apply this resource group to VMs running
                either ``RHEL`` or ``CentOS`` operating systems, specify 2
                items for the list with following values:
                inventory_filters[0].os_short_name='rhel' and
                inventory_filters[1].os_short_name='centos'

                If the list is empty, this resource group will be applied to
                the target VM unconditionally.
            resources (MutableSequence[google.cloud.osconfig_v1.types.OSPolicy.Resource]):
                Required. List of resources configured for
                this resource group. The resources are executed
                in the exact order specified here.
        """

        inventory_filters: MutableSequence[
            "OSPolicy.InventoryFilter"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="OSPolicy.InventoryFilter",
        )
        resources: MutableSequence["OSPolicy.Resource"] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="OSPolicy.Resource",
        )

    id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    description: str = proto.Field(
        proto.STRING,
        number=2,
    )
    mode: Mode = proto.Field(
        proto.ENUM,
        number=3,
        enum=Mode,
    )
    resource_groups: MutableSequence[ResourceGroup] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=ResourceGroup,
    )
    allow_no_resource_group_match: bool = proto.Field(
        proto.BOOL,
        number=5,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.cloud.osconfig.v1alpha",
    manifest={
        "OSPolicyComplianceState",
        "OSPolicyResourceConfigStep",
        "OSPolicyResourceCompliance",
    },
)


class OSPolicyComplianceState(proto.Enum):
    r"""Supported OSPolicy compliance states.

    Values:
        OS_POLICY_COMPLIANCE_STATE_UNSPECIFIED (0):
            Default value. This value is unused.
        COMPLIANT (1):
            Compliant state.
        NON_COMPLIANT (2):
            Non-compliant state
        UNKNOWN (3):
            Unknown compliance state.
        NO_OS_POLICIES_APPLICABLE (4):
            No applicable OS policies were found for the
            instance. This state is only applicable to the
            instance.
    """
    _pb_options = {"deprecated": True}
    OS_POLICY_COMPLIANCE_STATE_UNSPECIFIED = 0
    COMPLIANT = 1
    NON_COMPLIANT = 2
    UNKNOWN = 3
    NO_OS_POLICIES_APPLICABLE = 4


class OSPolicyResourceConfigStep(proto.Message):
    r"""Step performed by the OS Config agent for configuring an
    ``OSPolicyResource`` to its desired state.

    Attributes:
        type_ (google.cloud.osconfig_v1alpha.types.OSPolicyResourceConfigStep.Type):
            Configuration step type.
        outcome (google.cloud.osconfig_v1alpha.types.OSPolicyResourceConfigStep.Outcome):
            Outcome of the configuration step.
        error_message (str):
            An error message recorded during the
            execution of this step. Only populated when
            outcome is FAILED.
    """

    class Type(proto.Enum):
        r"""Supported configuration step types

        Values:
            TYPE_UNSPECIFIED (0):
                Default value. This value is unused.
            VALIDATION (1):
                Validation to detect resource conflicts,
                schema errors, etc.
            DESIRED_STATE_CHECK (2):
                Check the current desired state status of the
                resource.
            DESIRED_STATE_ENFORCEMENT (3):
                Enforce the desired state for a resource that
                is not in desired state.
            DESIRED_STATE_CHECK_POST_ENFORCEMENT (4):
                Re-check desired state status for a resource
                after enforcement of all resources in the
                current configuration run.
                This step is used to determine the final desired
                state status for the resource. It accounts for
                any resources that might have drifted from their
                desired state due to side effects from
                configuring other resources during the current
                configuration run.
        """
        _pb_options = {"deprecated": True}
        TYPE_UNSPECIFIED = 0
        VALIDATION = 1
        DESIRED_STATE_CHECK = 2
        DESIRED_STATE_ENFORCEMENT = 3
        DESIRED_STATE_CHECK_POST_ENFORCEMENT = 4

    class Outcome(proto.Enum):
        r"""Supported outcomes for a configuration step.

        Values:
            OUTCOME_UNSPECIFIED (0):
                Default value. This value is unused.
            SUCCEEDED (1):
                The step succeeded.
            FAILED (2):
                The step failed.
        """
        _pb_options = {"deprecated": True}
        OUTCOME_UNSPECIFIED = 0
        SUCCEEDED = 1
        FAILED = 2

    type_: Type = proto.Field(
        proto.ENUM,
        number=1,
        enum=Type,
    )
    outcome: Outcome = proto.Field(
        proto.ENUM,
        number=2,
        enum=Outcome,
    )
    error_message: str = proto.Field(
        proto.STRING,
        number=3,
    )


class OSPolicyResourceCompliance(proto.Message):
    r"""Compliance data for an OS policy resource.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        os_policy_resource_id (str):
            The id of the OS policy resource.
        config_steps (MutableSequence[google.cloud.osconfig_v1alpha.types.OSPolicyResourceConfigStep]):
            Ordered list of configuration steps taken by
            the agent for the OS policy resource.
        state (google.cloud.osconfig_v1alpha.types.OSPolicyComplianceState):
            Compliance state of the OS policy resource.
        exec_resource_output (google.cloud.osconfig_v1alpha.types.OSPolicyResourceCompliance.ExecResourceOutput):
            ExecResource specific output.

            This field is a member of `oneof`_ ``output``.
    """

    class ExecResourceOutput(proto.Message):
        r"""ExecResource specific output.

        Attributes:
            enforcement_output (bytes):
                Output from Enforcement phase output file (if
                run). Output size is limited to 100K bytes.
        """

        enforcement_output: bytes = proto.Field(
            proto.BYTES,
            number=2,
        )

    os_policy_resource_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    config_steps: MutableSequence["OSPolicyResourceConfigStep"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="OSPolicyResourceConfigStep",
    )
    state: "OSPolicyComplianceState" = proto.Field(
        proto.ENUM,
        number=3,
        enum="OSPolicyComplianceState",
    )
    exec_resource_output: ExecResourceOutput = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="output",
        message=ExecResourceOutput,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
